package com.freiheit.hibernate;

import com.freiheit.hibernate.beans.*;
import java.io.*;
import java.sql.*;
import java.util.*;
import jline.ConsoleReader;
import net.sf.hibernate.*;
import net.sf.hibernate.cfg.*;

public class Main {
    private static final String PROMPT = ">>> ";
    private static final String COMMAND_EXIT = "exit";

    SessionFactory _sessions;
    HsqldbManager _manager;
    
    public Main() throws ClassNotFoundException, HibernateException, MappingException, SQLException {
        Configuration configuration;
        Properties props;

        props = new Properties();
        props.setProperty( "hibernate.dialect", "net.sf.hibernate.dialect.HSQLDialect" );
        props.setProperty( "hibernate.show_sql", "true" );
        props.setProperty( "hibernate.use_outer_join", "true" );
        props.setProperty( "hibernate.query.substitutions", "true 1, false 0, yes 'Y', no 'N'" );
        
        configuration = new Configuration()
            .addClass( Address.class )
            .addClass( Bank.class )
            .addClass( CreditCardCompany.class )
            .addClass( PaymentMethod.class )
            .addClass( Country.class )
            .addClass( Customer.class )
            .addClass( Keyword.class )
            .addProperties( props );

        _sessions = configuration.buildSessionFactory();
        
        _manager = new HsqldbManager();
        _manager.createTestTables();
        _manager.addMasterData();
    }

    public HsqldbManager getManager() {
        return _manager;
    }
    
    public void createCustomerWithAddress() throws HibernateException, SQLException {
        Session session;
        Customer customer;
        Address address;
            
        System.out.println( "Creating customer with address..." );

        customer = new Customer();
        customer.setFirstname( "Hans" );
        customer.setLastname( "Wurst" );
        customer.setUsername( "hans@wurst.de" );
        customer.setPassword( "geheim" );

        address = new Address();
        address.setFirstname( "Hans" );
        address.setLastname( "Wurst" );
        address.setStreet( "Barstreet" );
        address.setNumber( "120a" );
        address.setPostalCode( "12345" );
        address.setCity( "Foostadt" );
        address.setCountry( new Country( 0 ) );
        
        customer.addAddress( address );
        
        session = _sessions.openSession( _manager.getConnection() );
        session.saveOrUpdate( customer );
        session.flush();
        session.connection().commit();
        
        System.out.println( _manager.dumpCustomerTable() );
        System.out.println( _manager.dumpAddressTable() );
    }
    
    public void createCustomerWithPaymentMethods() throws HibernateException, SQLException {
        Session session;
        Customer customer;
        BankAccount bankAccount;
        CreditCard creditCard;
            
        System.out.println( "Creating customer with bank-account and creditcard..." );

        customer = new Customer();
        customer.setFirstname( "Hans" );
        customer.setLastname( "Hansen" );
        customer.setUsername( "hans@hansen.de" );
        customer.setPassword( "trustnoone" );

        bankAccount = new BankAccount();
        bankAccount.setNumber( "12345" );
        bankAccount.setOwner( "Hans Hansen" );
        bankAccount.setBank( new Bank( 1 ) );
        
        creditCard = new CreditCard();
        creditCard.setNumber( "5212345678901234" );
        creditCard.setOwner( "Hans Hansen" );
        creditCard.setValidMonth( 1 );
        creditCard.setValidYear( 2009 );
        creditCard.setCreditCardCompany( new CreditCardCompany( 2 ) );

        customer.addBankAccount( bankAccount );
        customer.addCreditCard( creditCard );
        
        session = _sessions.openSession( _manager.getConnection() );
        session.saveOrUpdate( customer );
        session.flush();
        session.connection().commit();
        
        System.out.println( _manager.dumpCustomerTable() );
        System.out.println( _manager.dumpPaymentMethodTable() );
    }
    
    public void createCustomerWithKeyword() throws HibernateException, SQLException {
        Session session;
        Customer customer;
        Keyword keyword;
            
        System.out.println( "Creating customer with keyword..." );

        customer = new Customer();
        customer.setFirstname( "Peter" );
        customer.setLastname( "Petersen" );
        customer.setUsername( "peter@petersen.de" );
        customer.setPassword( "strenggeheim" );

        keyword = new Keyword( "Stammkunde" );
        customer.addKeyword( keyword );
        
        keyword = new Keyword( "Noch ein Schlsselwort" );
        customer.addKeyword( keyword );
        
        session = _sessions.openSession( _manager.getConnection() );
        session.saveOrUpdate( customer );
        session.flush();
        session.connection().commit();
        
        System.out.println( _manager.dumpCustomerTable() );
        System.out.println( _manager.dumpKeywordTable() );
        System.out.println( _manager.dumpKeywordCustomerRelationTable() );
    }
    
    public void printAllCountries() throws HibernateException {
        Session session;
        Query query;
        List result;
        
        System.out.println( "Loading all countries..." ); 
        
        session = _sessions.openSession( _manager.getConnection() );
        query = session.createQuery( "from Country country" );
        
        result = query.list();
        for ( int i = 0; i < result.size(); i++ ) {
            System.out.println( "\t" + result.get( i ) ); 
        }
    }
    
    public void printCustomersNamedHans() throws HibernateException {
        Session session;
        Query query;
        Iterator result;
        
        System.out.println( "Loading all customers called 'Hans'..." ); 
        
        session = _sessions.openSession( _manager.getConnection() );
        query = session.createQuery( "from Customer customer where customer.firstname='Hans'" );
        
        result = query.iterate();
        while ( result.hasNext() ) {
            System.out.println( "\t" + result.next() ); 
        }
    }
    
    public void printCustomersFromGermany() throws HibernateException {
        Session session;
        Query query;
        Iterator result;
        
        System.out.println( "Loading all customers from Germany..." ); 
        
        session = _sessions.openSession( _manager.getConnection() );
        query = session.createQuery( "select customer from Customer customer join customer.addresses address where address.country.name='Deutschland'" );
        
        result = query.iterate();
        while ( result.hasNext() ) {
            System.out.println( "\t" + result.next() ); 
        }
    }
    
    public void printCustomersWithSpecialCreditCard() throws HibernateException {
        Session session;
        Query query;
        Iterator result;
        
        System.out.println( "Loading all customers with an Eurocard..." ); 
        
        session = _sessions.openSession( _manager.getConnection() );
        query = session.createQuery( "select c from Customer as c join c.creditCards as card where card.creditCardCompany.name='Eurocard'" );
        
        result = query.iterate();
        while ( result.hasNext() ) {
            System.out.println( "\t" + result.next() ); 
        }
    }
    
    public void startConsole() {
        Session session;
        Query query;
        Iterator result;
        ConsoleReader reader;
        String command = "";

        try {
            reader = new ConsoleReader( System.in, new PrintWriter( System.out ) );

            while ( !command.equals( COMMAND_EXIT ) ) {
                command = reader.readLine( PROMPT );

                if ( ( command != null ) && !command.equals( COMMAND_EXIT ) ) {
                    try {
                        session = _sessions.openSession( _manager.getConnection() );
                        query = session.createQuery( command );

                        result = query.iterate();
                        while ( result.hasNext() ) {
                            System.out.println( "\t" + result.next() ); 
                        }
                    } catch ( HibernateException exception ) {
                        System.out.println( exception.getMessage() );
                    }
                } else {
                    command = COMMAND_EXIT;
                }
            }
        } catch ( IOException exception ) { /* exit */ }    

        System.out.println( "" ); 
    }
    
    public static void main( String[] args ) {
        Main hibernateDemo;
        boolean activateConsole;

        activateConsole = ( args.length == 1 ) && args[0].equals( "--console" );
        
        try {
            hibernateDemo = new Main();    
            
            System.out.println( "Content of all tables:" ); 
            System.out.println( hibernateDemo.getManager().dumpAllTables() );
            
            hibernateDemo.createCustomerWithAddress();
            hibernateDemo.createCustomerWithPaymentMethods();
            hibernateDemo.createCustomerWithKeyword();
            
            hibernateDemo.printAllCountries();
            hibernateDemo.printCustomersNamedHans();
            hibernateDemo.printCustomersFromGermany();
            hibernateDemo.printCustomersWithSpecialCreditCard();

            if ( activateConsole ) {
                hibernateDemo.startConsole();
            }
        } catch ( Exception exception ) {
            exception.printStackTrace();
        }
    }
}
